from constants import PENTAHO_USER_ENDPOINT_DEFINITION
import urlparse


class Pentaho(object):
    def __init__(self, *args, **kwargs):
        self.pentaho_base_url = kwargs.pop("pentaho_base_url", "http://localhost")
        self.pentaho_username = kwargs.pop("pentaho_username", "admin")
        self.pentaho_password = kwargs.pop("pentaho_password", "password")

    @staticmethod
    def validate_urls(url):
        """
        Validate a url
        :param url: the url to validate
        :return: True if valid, False otherwise
        """
        valid = False
        parsed_url = urlparse.urlparse(url)
        if parsed_url:
            valid = True
        return valid

    def get_user_urls(self, endpoint_type):
        """
        Generate the user based urls
        :param endpoint_type: the type of user endpoint required
        :return: the url for the endpoint
        """
        if not endpoint_type or endpoint_type not in PENTAHO_USER_ENDPOINT_DEFINITION:
            raise ValueError("[ERROR] get_user_urls::endpoint_type improper definition ... ")
        pentaho_user_endpoint = urlparse.urljoin(self.pentaho_base_url, PENTAHO_USER_ENDPOINT_DEFINITION[endpoint_type])
        if not self.validate_urls(pentaho_user_endpoint):
            raise ValueError("[ERROR] Improper user endpoint :: {} ... ".format(pentaho_user_endpoint))
        return pentaho_user_endpoint



# Pentaho endpoint constants



#############################################
# User Endpoint
#############################################

USERS_USERNAME_VAR = "userName"
USERS_PASSWORD_VAR = "password"

LIST_USERS = "list_users"
CREATE_USERS = "create_users"
DELETE_USERS = "delete_users"

PENTAHO_USER_ENDPOINT_DEFINITION = {
    CREATE_USERS: "/pentaho/api/userroledao/createUser",
    LIST_USERS: "/pentaho/api/users",
    DELETE_USERS: "/pentaho/api/userroledao/deleteUsers",
}



import logging
from pyexpat import ExpatError

import requests
import xmltodict as xmltodict

from constants import CREATE_USERS, LIST_USERS, USERS_USERNAME_VAR, USERS_PASSWORD_VAR

logger = logging.getLogger(__name__)


# TODO implement regex filtering
def list_all_users(pentaho=None, regex=None):
    """
    Get list of all users
    :param pentaho: the pentaho user object
    :return: (http/s Response, success/failure)
    :rtype: (String, list)
    """
    user_list = list()
    if not pentaho:
        raise ValueError("[ERROR] Pentaho object is missing ... ")
    response = requests.get(pentaho.get_user_urls(endpoint_type=LIST_USERS),
                            auth=(pentaho.pentaho_username, pentaho.pentaho_password))
    if response.status_code == 200:
        try:
            user_list = xmltodict.parse(response.text)['users']['user']
        except Exception, e:
            logger.exception(e)
    return response, user_list


def create_user(pentaho=None, username=None, password=None):
    """
    Create a user in pentaho
    :param pentaho: the pentaho object
    :param username: the username to assign to the user
    :param password: the password for the user
    :return: (http/s Response, success/failure )
    :rtype: (String, boolean)
    """
    created = False
    if not pentaho:
        raise ValueError("[ERROR] Pentaho object is missing ... ")
    if not username or not password:
        raise ValueError("[ERROR] user creation parameters missing ... ")
    user_credentials = {
        USERS_USERNAME_VAR: username,
        USERS_PASSWORD_VAR: password,
    }
    response = requests.put(pentaho.get_user_urls(endpoint_type=CREATE_USERS),
                            auth=(pentaho.pentaho_username, pentaho.pentaho_password),
                            json=user_credentials)
    if response.status_code == 200:
        created = True
    return response, created



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open


def get_requirements():
    """Simply read the requirements.txt file and returns the list of the dependencies.
    :returns: list of requirements
    """
    with open('requirements.txt') as f:
        requirements = f.read().splitlines()

    return requirements


reqs = get_requirements()

setup(
        name='pentaho-rest-api',

        # Versions should comply with PEP440.  For a discussion on single-sourcing
        # the version across setup.py and the project code, see
        # https://packaging.python.org/en/latest/single_source_version.html
        version='1.0.3',

        include_package_data=True,

        package_data={
            # If any package contains *.txt or *.rst files, include them:
            '': ['*.txt', '*.md'],
        },

        description='Python library for the pentaho BA REST API',
        long_description='Python library for the pentaho BA REST API',

        # The project's main homepage.
        url='git@github.com:AlayaCare/pentaho-rest-api.git',

        # Author details
        author='Saqib Khalil Yawar',
        author_email='saqib.yawar@alayacare.com',

        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Build Tools',
            'Programming Language :: Python :: 2.7',
            'License :: OSI Approved :: GNU General Public License (GPL)'
        ],

        # What does your project relate to?
        keywords='pentaho rest api development',

        packages=find_packages(exclude=['contrib', 'docs', 'tests']),

        install_requires=reqs,

        extras_require={
            'dev': reqs,
            'test': reqs,
        },

        # entry_points={
        #     'console_scripts': [
        #         'users_api=pentaho-rest-api.scripts.info:info',
        #     ],
        # },
)
